#version 100
#extension GL_OES_EGL_image_external : require //申明使用扩展纹理
precision highp float;
varying vec2 v_texcoord;
uniform sampler2D s_texture;
uniform vec2 u_texSize;
uniform highp float time;//时间,单位 s

#define PI 3.1415926535897932384626433832795
#define TAU 6.28318

const float A = 55.0;
const float B = 35.0;
const float C = 0.2;
const float D = 120.0;//控制数量
const float E = 10.0;
const float F = 0.06;

vec2 rotate(vec2 uv, float angle, float ratio, vec2 center) {
    uv -= center;
    mat2 m = mat2(vec2(ratio*cos(angle), -ratio*sin(angle)), vec2(sin(angle), cos(angle)));
    uv  = m * uv;
    uv.x /= ratio;

    uv += center;

    // uv += 1.0;
    // uv *= 0.5;
    // uv = fract(uv);
    // uv = abs((uv-0.5) * 2.0);

    return uv;
}

float random (in vec2 st) {
    return fract(sin(dot(st.xy,
    vec2(12.9898,78.233)))
    * 43758.5453123);
}

void main() {
    vec2 uv = v_texcoord;
    uv = 1.0 - uv;
    uv -= 0.5;
    uv = rotate(uv, - PI / 5.0, u_texSize.x / u_texSize.y, vec2(0.5));

    uv.y = 1.0 - uv.y;                      ///< 垂直翻转

    uv.x = uv.x * D;                        ///< 划分水平间隔，控制雨滴的个数
    float dx = fract(uv.x);                 ///< 每个间隔内的坐标
    uv.x = floor(uv.x);                     ///< 每个间隔的 id

    float speed = smoothstep(0.5, 1.0, random(vec2(uv.x + 300.0)));
    float t =  time * F * (1.0 + speed * 2.0) * 0.04;  ///< 控制速度

    uv.y *= C;                              ///< 雨滴的拉伸
    float o = sin(uv.x * 215.4);            ///< 水平不同位置雨滴的高度错位
    float s = cos(uv.x * 33.1) * .3 + .5;   ///< 单位时间下落的位移
    float trail = mix(A, B, s)*2.0;             ///< 控制了雨滴的拖尾长度

    float yv = fract(uv.y + t * s + o) * trail; ///< Magic， 随时间下落的矩形

    yv = 1.0 / (yv * yv);                       ///< 反色（其实是有除 0 的问题的，但是不影响效果）
    yv = smoothstep(0.0, 1.0, yv);              ///< 控制渐变

    yv = sin(yv * PI) * (s * E);                ///< 拖尾的渐变
    float d2 = sin(dx * PI);
    yv *= d2 * d2 * random(vec2(uv.x));

    float d = random(vec2(uv.x) + 20.0);

    yv *= step(0.9, d);

    vec3 col = vec3(1.0);
    col = col * yv;
    vec4 result = vec4(col, 1.0);

    vec4 backgroundCol = texture2D(s_texture, vec2(v_texcoord.x, 1.0-v_texcoord.y));

    gl_FragColor = result + backgroundCol;
}